 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#ifndef BINARYINSTANCE_H
#define BINARYINSTANCE_H

#include <vector>
#include <iostream>
#include <stdio.h>

/* This class defiens the representation of an individual as binary string. It implements several operators over
 * the individual's structure, and support fitness caching mechanism. */

using namespace std;

class BinaryInstance {
	unsigned int _size;			/* Individual's size.							*/
	bool _parent;				/* Parent boolean flag.							*/
	double _fitnessCache;			/* Individual's fitness cache value.					*/
	bool _validCache;			/* Fitness cache flag.							*/
	vector<bool> _individual;		/* Individual inner representation.					*/
public:
	/* Costructors and destructor. */
	BinaryInstance();
        BinaryInstance(int size);
	BinaryInstance(const BinaryInstance &bi);
        virtual ~BinaryInstance();

	static bool compare(BinaryInstance *i1, BinaryInstance *i2);
	
	/* Getters. */
	int get(unsigned int i);
	int getHarmonicEncoding(unsigned int i);
	bool getIsParent() {return _parent; };
	int getSize();
	double getFitnessCache();
	bool validCache();

	/* Setters. */
	void set(unsigned int i, int value);
	void setAllBits(int i);
	void setIsParent(bool parent) { _parent = parent; };
	void setFitnessCache(double fitness);
	
	/* Translators. */
	string toString();
	char* toCharArray();
};

#endif
