
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#ifndef STATISTICS_H
#define STATISTICS_H

#include "BinaryInstance.h"

using namespace std;

/* This class defines a statistics container, usuful to organize the statistics of a single generation execution
 * of an algorithm. */

class Statistics {
	
	unsigned int _generation; 		/* Value of the current generation. 					*/
	unsigned int _fitnessCalls;		/* Number of fitness calls. 						*/
	double _avgFitness; 			/* Average fitness of solution population.				*/
	double _bestInstance; 			/* Fitness value of the best instance in the population. 		*/
	double _worstInstance; 			/* Fitness value of the worst instance in the population. 		*/
	BinaryInstance *_bestIndividual;	/* Best individual.							*/
public:
        Statistics();
        Statistics(double avg, double best, double worst, int fitnessCalls);
        virtual ~Statistics();	
        
        /* Getters */
        unsigned int generation();
        unsigned int fitnessCalls();
        double avgFitness();
        double bestInstance();
        double worstInstance();
  	BinaryInstance* bestIndividual();      
	
	/* Setters */
	void setGeneration(unsigned int generation);
	void setFitnessCalls(unsigned int fitnessCalls);
	void setAvgFitness(double avg);
	void setBestInstance(double best);
	void setWorstInstance(double worst);
	void setBestIndividual(BinaryInstance *best);
};

#endif
