 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)       *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/
 
#include "EDA.h"

/* Class constructor. */
EDA::EDA(unsigned int popSize, const gsl_rng* rnd) : Algorithm() {
	_popSize = popSize;
	_rnd = rnd;
	parent_ = false;
}

/* Class copy constructor. */
EDA::EDA(const EDA &eda) {
	_popSize = eda._popSize;
	_rnd = eda._rnd;
	for (unsigned int i=0;i<eda._pop.size();i++) {
		_pop.push_back(new BinaryInstance(*eda._pop[i]));
	}
	parent_ = eda.parent_;
	Algorithm::setObjectiveFunction(eda);
}

/* Class destructor. */
EDA::~EDA() {
	unsigned int i; 
	
	for (i = 0; i < _pop.size(); i++) delete _pop[i];
	_pop.clear();
}

/* This function defines the single generation step of an algorithm. Can be overwrited if needed. */
void EDA::run() {
	sampling();
	selection();
	estimation();	
}

/* Prints a visual representation of the population. */
void EDA::printPopulation() {
	unsigned int i, j;
	
	for (i = 0; i < _pop.size(); i++) {
		printf("%3i: ",i+1);
		BinaryInstance* instance = _pop[i];
		for (j = 0; j < objectiveFunctionSize_; j++) {
			cout << instance->get(j);
		}
		printf(" (%.2f) \n",fitness(instance));
	}
	if (_pop.size()==0) {
		cout << "empty\n";
	}
}

/* This function computes and store the statistical values into a Statistics object,
 * and returns it. */
Statistics* EDA::computeStatistics() {
	int bindex;
	double avg = 0.0f, best = -1.0, worst = 1000.0, tmp;
	Statistics *stat;
	
	if (_pop.size() == 0) {
		stat = new Statistics(0,0,0,0);
		return stat;
	}
	
	for (unsigned int i=0;i<_pop.size();i++) {
			BinaryInstance* instance = _pop[i];
			tmp = fitness(instance);
			avg += tmp;
			if (tmp > best) { 
				best = tmp;
				bindex = i;
			}
			if (tmp < worst) worst = tmp;
	}
	avg /= _pop.size();
	stat = new Statistics(avg, best, worst, fitnessCalls());
	stat->setBestIndividual(new BinaryInstance(*_pop[bindex]));
	return stat;
}
