
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "IsingDEUM.h"

/* Class costructor. */
IsingDEUM::IsingDEUM(unsigned int pop_size, double cooling_rate, int sampler_iterations, const gsl_rng* rnd) :
	DEUM(pop_size, cooling_rate, sampler_iterations, rnd) {

}

/* Class costructor used from geda algorithm gDEUM*/
IsingDEUM::IsingDEUM(vector<BinaryInstance*> pop, Algorithm &algo, double cooling_rate, int sampler_iterations, const gsl_rng* rnd) :
	DEUM(pop.size(), cooling_rate, sampler_iterations, rnd) {
	Algorithm::setObjectiveFunction(algo);
	_pop = pop;
	setupStructure();
}

/* Class destructure. */
IsingDEUM::~IsingDEUM() {

}

/* Sets the MRF structure for the algorithm and initialize the coefficients vector. */
void IsingDEUM::setupStructure() {
	unsigned int i, l = sqrt(objectiveFunctionSize_);
	Clique *bivariate; 
	
	genBivariates(l);
	for (i = 0; i < objectiveFunctionSize_; i++) {
		bivariate = new Clique();
		bivariate->add(i);
		if (i % (l - 1) == 0 && i != 0)
			bivariate->add(i - l + 1);
		else
			bivariate->add(i + 1); 
		_mrf.push_back(bivariate);
	}
	_alpha = gsl_vector_alloc(_mrf.size() + 1);
}

/* Performs a single generation of the algorithm and returns the sampled individual. */
BinaryInstance* IsingDEUM::gedaGeneration() {
	BinaryInstance *sampled; 
	
	estimation();
	std::sort(_pop.begin(), _pop.end(), Comparator(this));
	sampled = gibbsKPCoolingSampler(_pop[0], _sampler_iterations, _cooling_rate);
	return sampled; 
}

/* Returns the algorithm name. */
string IsingDEUM::getName() {
	return "IsingDEUM";
}
