
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#ifndef MIMIC_H_
#define MIMIC_H_

#include <list>
#include <float.h>

#include "EDA.h"

using namespace std;

class MIMIC : public EDA {

	protected:
		class RingOfChain {
			int _permutation_index;
			double _entropy; 
			double _probability;
			
			public: 
				RingOfChain(int index, double entropy): _permutation_index(index), _entropy(entropy) {};
				virtual ~RingOfChain() {};
				
				int getIndex() { return _permutation_index; };
				double getEntropy() { return _entropy; };
				double getProbability() { return _probability; };
				void setProbability(double prob) { _probability = prob; };
		};

		list<RingOfChain*> _chain;
		int _offspring;

	public:
		MIMIC(int popSize, int offspring, const gsl_rng* rnd);
		virtual ~MIMIC();

		void sampling();
		void selection();
		void estimation();
		
		void run();
		string getName();
		void setObjectiveFunction(ObjectiveFunction* of);
	private:
		double entropy(double probability);
		double conditionalEntropy(int index);
		double conditionalProbability(int index_target, int index_know, int know_value);
};

#endif
