 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "PBIL.h"

/* Class constructor. */
PBIL::PBIL(unsigned int popSize, double gamma, const gsl_rng* rnd) : UnivariateEDA(popSize, rnd) {
 	_gamma = gamma;
}

/* Class constructor with a given probability vector. */
PBIL::PBIL(unsigned int popSize, vector<double> prob, double gamma, const gsl_rng* rnd) : UnivariateEDA(popSize, prob, rnd) {
 	_gamma = gamma;
}

/* Class destructor. */
PBIL::~PBIL() {
	
}

/* Updates the probability vector according to the current population univariate distribution. */
void PBIL::estimation() {
	unsigned int i, j;
	double sum;
	
	for (j = 0; j < _prob.size(); j++) {
		sum = 0;
		for (i = 0; i < _pop.size(); i++) {
			sum += _pop[i]->get(j);
		}
		_prob[j] = (1-_gamma)*_prob[j] + _gamma*(sum/_pop.size());
	}	
}

/* Return the algorithm name. */
string PBIL::getName() {
	return "PBIL";
}
