
 /*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/
 
#ifndef GA_H_
#define GA_H_

#include <algorithm>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

#include "Algorithm.h"
#include "Statistics.h"

using namespace std;

/* This class defines the abstract class for a Genetic Algorithm. Every class that inherits from GA must
 * defines at least selection, crossover, mutation operators and the function run. */
class GA : public Algorithm {

protected:
	unsigned int _popSize;			/* Population size. 					*/
	double _mutRate;			/* Mutation probability.				*/
	const gsl_rng* _rnd;			/* Random number generator reference.			*/
	vector< BinaryInstance* > _pop;		/* Population container.				*/

public:
	GA(unsigned int popSize, double mutRate, const gsl_rng* rnd);
	virtual ~GA();

	void setObjectiveFunction(ObjectiveFunction* of);
	void printPopulation();
	Statistics* computeStatistics();
	
	virtual void selection() = 0; 
	virtual void crossover() = 0; 
	virtual void mutation() = 0;  
	virtual void run() = 0;
};
#endif
