 
 /*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/
 
#include "SGA.h"
#include "Comparator.h"

using namespace std;

/* Class constructor. */
SGA::SGA(int popSize, double mutRate, const gsl_rng* rnd) : GA(popSize, mutRate, rnd) {

}

/* Class destructor. */
SGA::~SGA() {

}

/* This function defines the generation step. */
void SGA::run() {
	selection();
	crossover();
	mutation();
}


/* Truncation Selection, deletes the worst 50% of population. */
void SGA::selection() {	
	unsigned int i;
	
	std::sort(_pop.begin() , _pop.end(), Comparator(this));
	for (i= _pop.size() / 2; i < _pop.size(); i++) delete _pop[i];
	_pop.erase(_pop.begin() + _pop.size() / 2, _pop.end());
}

/* One-point Crossover, generates new offspring through one-point crossover. The amount of offspring
 * generated depends on the selection. This operators fill the selected population with new offstring 
 * until it reaches the original population size. */
void SGA::crossover() {
	vector<BinaryInstance*> newPop;
	unsigned int j , crossPoint;
	BinaryInstance *parent1, *parent2;
	BinaryInstance *child1, *child2;	
	int p1, p2;
	
	do {		
		child1 = new BinaryInstance(objectiveFunctionSize_);
		child2 = new BinaryInstance(objectiveFunctionSize_);
		
		/* Chooses two individuals randomly. */
		p1 = gsl_rng_uniform_int(_rnd, _pop.size());
		p2 = gsl_rng_uniform_int(_rnd, _pop.size());
		
		parent1 = _pop[p1];
		parent2 = _pop[p2];

		/* One-Point Crossover*/
 		crossPoint = gsl_rng_uniform_int(_rnd,objectiveFunctionSize_);
  		for (j = 0; j < crossPoint; j++) {
 			child1->set(j,parent1->get(j));
 			child2->set(j,parent2->get(j));
 		}
 		
 		for (j = crossPoint; j < objectiveFunctionSize_; j++) {
 			child1->set(j,parent2->get(j));
 			child2->set(j,parent1->get(j));
 		}
 		
		newPop.push_back(child1);
		newPop.push_back(child2);
	} while (newPop.size() + _pop.size()< _popSize);

	_pop.insert(_pop.end(), newPop.begin(), newPop.end());
	newPop.clear();
}

/* Bit-Flip Mutation. For each individual in the population, for each alle of the individual, 
   flips the current allele value with probability _mutRate. */
void SGA::mutation() {
	unsigned int i,j;
	
	for (i = 0; i < _pop.size(); i++) {
		for (j=0;j<objectiveFunctionSize_;j++) {
			if (gsl_ran_bernoulli(_rnd, _mutRate)) {
				if (_pop[i]->get(j)==0) 
					_pop[i]->set(j,1);
				else
					_pop[i]->set(j,0);
			}
		}
	}
}

/* Returns the algorithm name. */
string SGA::getName() {
	return "SGA";
}
