
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "SGABinTourn.h"

/* Class constructor. */
SGABinTourn::SGABinTourn(int popSize, double mutRate, const gsl_rng* rnd) : SGA(popSize, mutRate, rnd) {	

}

/* Class destructor. */
SGABinTourn::~SGABinTourn() {

}

/* This function defines the generation step. */
void SGABinTourn::run() {
	selection();
	crossover();
	mutation();
}

/* Binary Tournament selection, it deletes the 50% of the population. */
void SGABinTourn::selection() {
        vector< BinaryInstance* > newPop;
	int n1, n2;
	
	// Select the 50% of population 
	for (unsigned int i=0; i<_pop.size()/2; i++) {
		BinaryInstance *instance1, *instance2, *selected;
		
		n1 = gsl_rng_uniform_int(_rnd,_pop.size());
		n2 = gsl_rng_uniform_int(_rnd,_pop.size() - 1);
		if (n2 >= n1) n2++;
		
		instance1 = _pop[n1];
		instance2 = _pop[n2];
		
		if(fitness(instance1) > fitness(instance2))
			selected = new BinaryInstance(*instance1);
		else 
			selected = new BinaryInstance(*instance2);
			
		newPop.push_back(selected);
	}
	
	// Delete the whole old population
	for (unsigned int i=0; i<_pop.size(); i++) {
		delete _pop[i];
	}
	_pop.clear();
	
	_pop.insert(_pop.end(), newPop.begin(), newPop.end());
	newPop.clear();
}

/* Returns the algorithm name. */
string SGABinTourn::getName() {
	return "SGABinaryTournment";
}

