 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "AlgorithmDecorator.h"

/* Class constructor. */
AlgorithmDecorator::AlgorithmDecorator(Glib::ustring &title) {
	_title = title; 
	initialize(); 
}

/* Class destructor. */
AlgorithmDecorator::~AlgorithmDecorator() {

}

/* True if the algorithm is selected, false otherwise. */
bool AlgorithmDecorator::isActive() {
	return _ck_btn->get_active();
}

/* Set the sensitivity of the widget. */
void AlgorithmDecorator::setSensitive(bool mode) {
	this->set_sensitive(mode);
}

/* Partially initialization of the decorator. */
void AlgorithmDecorator::initialize() { 
	_ck_btn = new Gtk::CheckButton();
	_expander = new Gtk::Expander();
	_vbox = new Gtk::VBox();
	
	_ck_btn->set_label("");
	_expander->set_label(_title);
	_expander->add(*_vbox);
	
	this->pack_start(*_ck_btn, false, false); 
	this->pack_start(*_expander); 
}

/* This function adds an input entry to the decorator. */
void AlgorithmDecorator::addParameter(ParamType type) {
	Gtk::Label *label;
	Gtk::SpinButton *btn;
	Gtk::HBox *box; 
	
	label = new Gtk::Label();
	btn = new Gtk::SpinButton(); 
	box = new Gtk::HBox(); 
	
	_label.push_back(label);
	_sp_btns.push_back(btn);
	_hbox.push_back(box);
	
	btn->set_size_request(85,-1);
	
	box->pack_start(*label);
	box->pack_start(*btn, Gtk::PACK_SHRINK);
	
	_vbox->pack_start(*box); 
	  
	switch (type) {
		case  POP :
			label->set_label("Population Size: ");
			
			btn->set_digits(0);
			btn->set_increments(10,10);
			btn->set_range(50, 10000);
			btn->set_value(100);
			break;
			
		case  GEDA_POP :
			label->set_label("gEDA Population Size: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(2, 5000);
			btn->set_value(8);
			break;
			
		case  EDA_POP :
			label->set_label("EDA Population Size: ");
			
			btn->set_digits(0);
			btn->set_increments(10,10);
			btn->set_range(50, 1000);
			btn->set_value(100);
			break;
			
		case  MUT_RATE :
			label->set_label("Mutation Probability: ");
			
			btn->set_digits(3);
			btn->set_increments(0.001,0.001);
			btn->set_range(0,1);
			btn->set_value(0.002);
			break;
			
		case  GAMMA :
			label->set_label("Gamma: ");
			
			btn->set_digits(3);
			btn->set_increments(0.001,0.001);
			btn->set_range(0,1);
			btn->set_value(0.998);
			break;
			
		case  FACTOR :
			label->set_label("Factor/Iterations: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(1, 5000);
			btn->set_value(1);
			break;
			
		case  EPSILON :
			label->set_label("Epsilon: ");
			
			btn->set_digits(2);
			btn->set_increments(0.01,0.01);
			btn->set_range(0,1);
			btn->set_value(0.01);
			break;
		
		case  MAX_INCOMING :
			label->set_label("Max incoming: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(0,50);
			btn->set_value(4);
			break;
			
		case  PARENT_PERCENTAGE :
			label->set_label("Parent percentage: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(1,99);
			btn->set_value(50);
			break;
		
		case  OFFSPRING :
			label->set_label("Offspring Size: ");
			
			btn->set_digits(0);
			btn->set_increments(10,10);
			btn->set_range(50, 10000);
			btn->set_value(100);
			break;
		
		case  NUM_SAMPLES :
			label->set_label("Number of Samples: ");
			
			btn->set_digits(0);
			btn->set_increments(10,10);
			btn->set_range(10, 500);
			btn->set_value(50);
			break;
		
		case  PERCENTAGE_SELECTION :
			label->set_label("Selection percentage: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(1,100);
			btn->set_value(50);
			break;
		
		case  MAX_INFLUENCE :
			label->set_label("Max Influence: ");
			
			btn->set_digits(0);
			btn->set_increments(10,10);
			btn->set_range(10, 1000);
			btn->set_value(10);
			break;
		
		case  FAST_SEARCH_ITERATIONS :
			label->set_label("Fast-Search iterations: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(1, 50);
			btn->set_value(1);
			break;
		
		case  COOLING_RATE :
			label->set_label("Cooling rate: ");
			
			btn->set_digits(5);
			btn->set_increments(0.00005,10);
			btn->set_range(0.00005,1);
			btn->set_value(0.9);
			break;
		
		case  DEPTH :
			label->set_label("Maximum tree depth: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(2, 10);
			btn->set_value(4);
			break;
		
		case  CROSS_RATE :
			label->set_label("Crossover probability: ");
			
			btn->set_digits(3);
			btn->set_increments(0.001,0.001);
			btn->set_range(0,1);
			btn->set_value(0.900);
			break;
		case  CHAIN_OFFSET :
			label->set_label("Chain offset: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(1, 1000);
			btn->set_value(1);
			break;
		
		case  MAX_CLIQUE_SIZE :
			label->set_label("Max clique size: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(1, 1000);
			btn->set_value(3);
			break;
		
		case  MAX_NUM_OF_CLIQUE :
			label->set_label("Max cliques number: ");
			
			btn->set_digits(0);
			btn->set_increments(1,1);
			btn->set_range(1, 1000);
			btn->set_value(3);
			break;
	}
}
