
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "EvoptoolGUI.h"

/*-----------------------------------------------------------------------------------------------------------------*/
/*						PUBLIC FUNCTIONS 						   */
/*-----------------------------------------------------------------------------------------------------------------*/

/* Class constructor. */
EvoptoolGUI::EvoptoolGUI(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade) :
	Gtk::Window(cobject),
	_refGlade(refGlade) {
	gdk_threads_init();
	initializeWidgetsAttributes();
}

/* Class destructor. */
EvoptoolGUI::~EvoptoolGUI() {
	_resDecorators.clear();
	_needSquareSizeResDecorators.clear();
}

/* Sets the pointer to the underline Evoptool engine and call the signals connector. */
void EvoptoolGUI::setEvoptoolReference(Evoptool *tool) {
	_evoptool = tool;
	connectSignalHandlers();
}

/*----------------------------------------------------------------------------------------------------------------*/
/* 						PRIVATE FUNCTIONS			       			  */
/*----------------------------------------------------------------------------------------------------------------*/
/* ------------------------------------------ Initializer functions --------------------------------------------- */

/* Graphic user interface initializer. */
void EvoptoolGUI::initializeWidgetsAttributes() {
	initializeLogVisualizer();
	initializeStatisticsVisualizer();
	initializeResolutorsPanel();
	initializeTaskBar();
	initializeCommandBar();
	initializeMainWindow();
}

/* Initializes main window properties. */
void EvoptoolGUI::initializeMainWindow() {
	set_title("evoptool - Evolutive Optimization Tool");
	set_position(Gtk::WIN_POS_CENTER);
	set_default_size(1000,600);
	maximize();
}

/* Initializes command bar properties. */
void EvoptoolGUI::initializeCommandBar() {
	char *user = getenv("USER");
	char path[100];
	sprintf(path, "/home/%s/", user);
	
	_refGlade->get_widget("commandbar", _commandBar);
	_refGlade->get_widget("runbutton", _runButton);
	_refGlade->get_widget("stopbutton", _stopButton);
	_stopButton->set_sensitive(false);
	_refGlade->get_widget("savebutton", _saveButton);
	_refGlade->get_widget("modebutton", _modeButton);
	_refGlade->get_widget("folderbutton", _folderButton);
	_folderButton->set_action(Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
	_folderButton->set_current_folder(path);
	_refGlade->get_widget("archiveentry", _archiveEntry);
	_archiveEntry->select_region(0,-1);
	_refGlade->get_widget("averagebutton", _averageButton);
	_refGlade->get_widget("generationsbutton", _generationsButton);
	_refGlade->get_widget("statusbar", _statusBar);
	_statusBar->push("Evoptool: idle..");
}

/* Initializes task bar properties. */
void EvoptoolGUI::initializeTaskBar() {
	const int spinGlassBoxLen = 9;	
	const int taskBoxLen = 16;
	const int textureRestorationBoxLen = 9;
	
	_refGlade->get_widget("sizespinbutton", _sizeSpinButton);
	_refGlade->get_widget("genspinbutton", _genSpinButton);
	_refGlade->get_widget("gencombobox", _genComboBox);
	_refGlade->get_widget("sizelabel", _sizeLabel);
	_refGlade->get_widget("genlabel", _genLabel);
	_refGlade->get_widget("taskbar", _taskBar);
	
	_refGlade->get_widget("taskbox", _taskBox);
	BoxEntry taskBoxEntry[taskBoxLen] = {
		{(char*) "Alternate Bits", Evoptool::ALT_BITS},
		{(char*) "Binary Value", Evoptool::BIN_VAL},
		{(char*) "F3D Deceptive", Evoptool::F3D_DECEPTIVE},
		{(char*) "F3D Deceptive Bipolar", Evoptool::F3D_DECEPTIVE_BIPOLAR},
		{(char*) "F3D Deceptive Overlapping", Evoptool::F3D_DECEPTIVE_OVERLAPPING},
		{(char*) "Four Peaks", Evoptool::FOUR_PEAKS},
		{(char*) "Liepins Vose Fully Deceptive", Evoptool::LIEPINS_VOSE_FULLY_DECEPTIVE},
		{(char*) "One Max", Evoptool::ONE_MAX},
		{(char*) "One Zero Max", Evoptool::ONE_ZERO_MAX},
		{(char*) "Quadratic", Evoptool::QUADRATIC},
		{(char*) "Max Satisfiability", Evoptool::MAX_SAT},
		{(char*) "Six Peaks", Evoptool::SIX_PEAKS},
		{(char*) "Sum Value", Evoptool::SUM_VAL},
		{(char*) "Trap 5", Evoptool::TRAP5},
		{(char*) "Spin Glass", Evoptool::SPIN_GLASS},
		{(char*) "Texture Restoration", Evoptool::TEXTURE_RESTORATION},
	};
	
	createTreeModel(taskBoxEntry, taskBoxLen, _taskTreeModel);
	_taskBox->set_model(_taskTreeModel);
	_taskBox->pack_start(_modelColumns.mcId);  
	_taskBox->pack_start(_modelColumns.mcText);
		
	BoxEntry spinGlassBoxEntry[spinGlassBoxLen] = {
		{(char*) "3x3", 9},
		{(char*) "4x4", 16},
		{(char*) "5x5", 25},
		{(char*) "8x8", 64},
		{(char*) "9x9", 81},
		{(char*) "10x10", 100},
		{(char*) "13x13", 169},
		{(char*) "14x14", 196},
		{(char*) "15x15", 225},
	};

	createTreeModel(spinGlassBoxEntry, spinGlassBoxLen, _spinGlassTreeModel);
	
	BoxEntry textureRestorationBoxEntry[textureRestorationBoxLen] = {
		{(char*) "chess30x30.png", 1},
		{(char*) "floor30x30.png", 2},
		{(char*) "sponge30x30.png", 3},
		{(char*) "chess60x60.png", 4},
		{(char*) "floor60x60.png", 5},
		{(char*) "sponge60x60.png", 6},
		{(char*) "chess80x80.png", 7},
		{(char*) "floor80x80.png", 8},
		{(char*) "sponge80x80.png", 9},
	};

	createTreeModel(textureRestorationBoxEntry, textureRestorationBoxLen, _textureRestorationTreeModel);
	_genComboBox->pack_start(_modelColumns.mcId);  
	_genComboBox->pack_start(_modelColumns.mcText);
}

/* Initializes and adds resolutor decorators to their respective box. */
void EvoptoolGUI::initializeResolutorsPanel() {
	unsigned int i;
	vector<AlgorithmDecorator*> tmp;
	Gtk::VBox *vbox; 
	
	_refGlade->get_widget("algorithmswindow", _resolutorsPanel);
	
	/*Initializing GA Decorators*/
	tmp.push_back(new SGADecorator(*(new Glib::ustring("SGA"))));
	tmp.push_back(new SGABinTournDecorator(*(new Glib::ustring("SGA Binary Tournment"))));
	tmp.push_back(new SGAPopWiseCrossDecorator(*(new Glib::ustring("SGA Pop-Wise Uniform Crossover"))));
	
	_refGlade->get_widget("gavbox", vbox);	
	for (i = 0; i < tmp.size(); i++) {
		vbox->add(*tmp[i]);
		manage(tmp[i]);
	}
	
	vbox->show_all();
	_resDecorators.insert(_resDecorators.end(), tmp.begin(), tmp.end());
	tmp.clear();	
	
	/*Initializing EDA Decorators*/
	tmp.push_back(new cGADecorator(*(new Glib::ustring("cGA"))));
	tmp.push_back(new PBILDecorator(*(new Glib::ustring("PBIL"))));
	tmp.push_back(new UMDADecorator(*(new Glib::ustring("UMDA"))));
	tmp.push_back(new MIMICDecorator(*(new Glib::ustring("MIMIC"))));
	tmp.push_back(new pbilCOMITDecorator(*(new Glib::ustring("pbilCOMIT"))));
	tmp.push_back(new hcCOMITDecorator(*(new Glib::ustring("hcCOMIT"))));
	tmp.push_back(new UnivariateDEUMDecorator(*(new Glib::ustring("UnivariateDEUM"))));
	_needSquareSizeResDecorators.push_back(tmp.back());
	tmp.push_back(new BivariateDEUMDecorator(*(new Glib::ustring("BivariateDEUM"))));
	_needSquareSizeResDecorators.push_back(tmp.back());
	tmp.push_back(new IsingDEUMDecorator(*(new Glib::ustring("IsingDEUM"))));
	_needSquareSizeResDecorators.push_back(tmp.back());
	//tmp.push_back(new sBOADecorator(*(new Glib::ustring("sBOA"))));
	
	_refGlade->get_widget("edavbox", vbox);	
	for (i = 0; i < tmp.size(); i++) {
		vbox->add(*tmp[i]);
		manage(tmp[i]);
	}
	
	vbox->show_all();
	_resDecorators.insert(_resDecorators.end(), tmp.begin(), tmp.end());
	tmp.clear();	
		
}

/* Initialize and add simple graph tabs to statisticsBook. This basic graphs are always present and 
 * they do not depend on the particular task. */
void EvoptoolGUI::initializeStatisticsVisualizer() {
	SimpleGraphTab *tab;
	
	_refGlade->get_widget("hpaned", _hpaned);
	_refGlade->get_widget("statisticsbook", _statisticsBook);
	_refGlade->get_widget("statisticsviewport", _statisticsViewport);

	_previousTab = 0;
	_hpanedPosition = _hpaned->get_position();
	_statisticsHandlerEffectiveness = false;
	
	/* Adding basic graphs. */
	tab = Gtk::manage(new SimpleGraphTab(*new Glib::ustring("Average/Generations"),
					     *new string(AVG_IMAGE_FILE), *new string(AVG_GNUPLOT_FILE)));
	_statisticsBook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*tab, tab->tabName()));

	tab = Gtk::manage(new SimpleGraphTab(*new Glib::ustring("Best/Generations"),
					     *new string(BEST_IMAGE_FILE), *new string(BEST_GNUPLOT_FILE)));
	_statisticsBook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*tab, tab->tabName()));

	tab = Gtk::manage(new SimpleGraphTab(*new Glib::ustring("Worst/Generations"),
					     *new string(WORST_IMAGE_FILE), *new string(WORST_GNUPLOT_FILE)));
	_statisticsBook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*tab, tab->tabName()));

	tab = Gtk::manage(new SimpleGraphTab(*new Glib::ustring("Average/FitnessCalls"),
					     *new string(FCALLS_IMAGE_FILE), *new string(FCALLS_GNUPLOT_FILE)));
	_statisticsBook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*tab, tab->tabName()));

	_statisticsBook->show_all_children();
}

/* Initialize logger references. */
void EvoptoolGUI::initializeLogVisualizer() {
	_refGlade->get_widget("logview", _logView);
	_logBuffer = _logView->get_buffer();
}
	
/* --------------------------------------- Signal handlers functions -------------------------------------------- */

/* Signal handlers connector. */
void EvoptoolGUI::connectSignalHandlers() {
	_runButton->signal_clicked().connect(sigc::mem_fun(*this, &EvoptoolGUI::runButtonHandler));
	_stopButton->signal_clicked().connect(sigc::mem_fun(*this, &EvoptoolGUI::stopButtonHandler));
	_taskBox->signal_changed().connect( sigc::mem_fun(*this, &EvoptoolGUI::taskBoxHandler));
	_statisticsBook->signal_switch_page().connect( sigc::mem_fun(*this, &EvoptoolGUI::statisticsBookHandler));
	_hpaned->signal_size_allocate().connect(sigc::mem_fun(*this, &EvoptoolGUI::hpanedHandler));
	_evoptool->signalRunTest().connect(sigc::mem_fun(*this, &EvoptoolGUI::signalRunTestHandler));
	_evoptool->signalFinalizeTest().connect(sigc::mem_fun(*this, &EvoptoolGUI::signalFinalizeTestHandler));
	_evoptool->signalTestExecuted().connect(sigc::mem_fun(*this, &EvoptoolGUI::signalTestExecutedHandler));
	_logHandler = _evoptool->signalNewLog().connect(sigc::mem_fun(*this, &EvoptoolGUI::signalNewLogHandler));
}

/* Updates GUI behaviors according to event received. */
void EvoptoolGUI::signalRunTestHandler() {
	gdk_threads_enter();
	
	sigc::slot<bool> mySlot = sigc::bind(sigc::mem_fun(*this, &EvoptoolGUI::statisticsRefreshTimer), 1);
	_statisticsRefreshTimer = Glib::signal_timeout().connect(mySlot, STATISTICS_UPDATE_TIME);
	_stopButton->set_sensitive(true);
	_statusBar->push("Evoptool: running test..");
	
	gdk_threads_leave();
}

/* Updates GUI behaviors according to event received. */
void EvoptoolGUI::signalFinalizeTestHandler() {
	gdk_threads_enter();
	
	_statisticsRefreshTimer.disconnect();
	_stopButton->set_sensitive(false);
	_statusBar->push("Evoptool: finalizing test..");
	
	gdk_threads_leave();
}

/* Restores the original GUI behavior after the test execution. */
void EvoptoolGUI::signalTestExecutedHandler() {
	gdk_threads_enter();

	_runButton->set_sensitive(true);			
	_modeButton->set_sensitive(true);		
	_averageButton->set_sensitive(true);		
	_generationsButton->set_sensitive(true);	
	_taskBar->set_sensitive(true);
	_resolutorsPanel->set_sensitive(true);	
	_statusBar->push("Evoptool: test executed..");
	updateStatisticsVisualization();
	
	gdk_threads_leave();
}

/* New log handler, update the log visualization. */		/* TURNED OFF - NOT STABLE */
void EvoptoolGUI::signalNewLogHandler(string message) {
//	_logHandler.disconnect();
//	_logIterator = _logBuffer->end();
//	_logBuffer->insert(_logIterator, message);
//	_logView->scroll_to(_logBuffer->get_mark("insert"));
//	_logHandler = _evoptool->signalNewLog().connect(sigc::mem_fun(*this, &EvoptoolGUI::signalNewLogHandler));
}

/* This handler check for parameters consistency and signals to start the test initialization phase. */
void EvoptoolGUI::runButtonHandler() {
	_runButton->set_sensitive(false);		
	_stopButton->set_sensitive(false);			
	_modeButton->set_sensitive(false);		
	_averageButton->set_sensitive(false);		
	_generationsButton->set_sensitive(false);			
	_taskBar->set_sensitive(false);
	_resolutorsPanel->set_sensitive(false);
	
	if (taskValidAssignment() && resolutorsValidAssignment() && exportValidAssignment()) {
		_statisticsHandlerEffectiveness = true;
		_statusBar->push("Evoptool:: initializing test parameters..");
		Glib::Thread::create(sigc::mem_fun(*this, &EvoptoolGUI::initializerThread), false);
	} else {
		_runButton->set_sensitive(true);		
		_stopButton->set_sensitive(false);			
		_modeButton->set_sensitive(true);		
		_averageButton->set_sensitive(true);		
		_generationsButton->set_sensitive(true);
		_taskBar->set_sensitive(true);
		_resolutorsPanel->set_sensitive(true);	
	}
}

/* This handler take care to propagate the stopButton signal to Evoptool by emitting a signalStopTest. */
void EvoptoolGUI::stopButtonHandler() {
	_stopButton->set_sensitive(false);
	_evoptool->signalStopTest().emit();
}

/* This handler takes care about loading the correct input parameter for the previously selected task. */
void EvoptoolGUI::taskBoxHandler() {
	Gtk::TreeModel::iterator it;
	Gtk::TreeModel::Row row;
	int id;
	
	it = _taskBox->get_active();
	row = *it;
	
	_sizeSpinButton->hide();
	_genSpinButton->hide();
	_genComboBox->hide();
	_sizeLabel->hide();
	_genLabel->hide();
		
	if (it && row) {
		id = row[_modelColumns.mcId];
		switch(id) {
			case -1: break;
			case Evoptool::FOUR_PEAKS: 
			case Evoptool::SIX_PEAKS: 
				_sizeSpinButton->show();
				_genSpinButton->show();
				_sizeLabel->show();
				_genLabel->set_label(" Peaks: ");
				_genLabel->show();
				break;	
			case Evoptool::MAX_SAT : break;
			case Evoptool::SPIN_GLASS : 
				_sizeLabel->show();
				_genComboBox->unset_model();
				_genComboBox->set_model(_spinGlassTreeModel);
				_genComboBox->show();
				break;
			case Evoptool::TEXTURE_RESTORATION: 
				_genLabel->set_label(" Image: ");
				_genLabel->show();
				_genComboBox->unset_model();
				_genComboBox->set_model(_textureRestorationTreeModel);
				_genComboBox->show();
				break;		
			default: 
				_sizeSpinButton->show();
				_sizeLabel->show();
				break;
		}	
	}
}

/* Statistics refresh timer handler, takes care about repaint the current visualized page inside 
 * statistics book. */
bool EvoptoolGUI::statisticsRefreshTimer(int timerNumber) {
	gdk_threads_enter();
	updateStatisticsVisualization();
	gdk_threads_leave();
	
	return true;
}

/* Signal handler for statistics notebook, takes care about repaint the current visualized page 
 * inside statistics book. */
void EvoptoolGUI::statisticsBookHandler(GtkNotebookPage* page, guint page_num) {
	if (_statisticsHandlerEffectiveness) updateStatisticsVisualization();
}

/* Signal handler for hpaned, parent container of statistics book, takes care about repaint the 
 * current visualized page  inside statistics book. */
void EvoptoolGUI::hpanedHandler(Gdk::Rectangle &allocation) {
	int tmp = _hpaned->get_position();
 
	if (_hpanedPosition != tmp) {
		_hpanedPosition = tmp; 
		if (_statisticsHandlerEffectiveness) updateStatisticsVisualization();
	}
}

/* ------------------------------------- Test initialization functions ------------------------------------------- */

/* Creates and fills a TestParameters structure and send it to Evoptool throught signalInitializeTest signal. */
void EvoptoolGUI::initializerThread() {
	Evoptool::TestParameters *_params;
	
	_params = new Evoptool::TestParameters();
	initializeConfiguration(_params);	
	initializeTask(_params);
	initializeResolutors(_params);
	gdk_threads_enter();
	initializeStatisticsVisualization(_params);
	gdk_threads_leave();
	_evoptool->signalInitializeTest().emit(_params);
}

/* Initializes configuration parameters. */
void EvoptoolGUI::initializeConfiguration(Evoptool::TestParameters *params) {
	unsigned int i;
	
	params->exportData = _saveButton->get_active();
	if (params->exportData) {
		params->destinationFolder = new char[100];
		params->archiveName = new char[100];
		sprintf(params->destinationFolder, "%s/", _folderButton->get_current_folder().c_str());
		sprintf(params->archiveName, "%s" , _archiveEntry->get_text().c_str());
	}	
	
	params->averaged = _modeButton->get_active();
	if (params->averaged) params->averageSamples = _averageButton->get_value_as_int();
	else params->averageSamples = 1;
	
	params->perfectSquareSize = false;
	for (i = 0; i < _needSquareSizeResDecorators.size(); i++) {
		if ( _needSquareSizeResDecorators[i]->isActive())
			params->perfectSquareSize = true;
	}
	
	params->maxGenerations = _generationsButton->get_value_as_int();
	params->statisticsOptions = Evoptool::BASIC_GRAPHS;
	params->rng = _evoptool->getRNG();
	params->mutex = new Glib::Mutex();
	params->runCondition = true;

}

/* Initializes task function. */
void EvoptoolGUI::initializeTask(Evoptool::TestParameters *params) {
	int taskId, size;
	Gtk::TreeModel::Row taskRow, genRow;
	
	taskRow = *(_taskBox->get_active());
	taskId = taskRow[_modelColumns.mcId];
	size = _sizeSpinButton->get_value_as_int();
	if (params->perfectSquareSize) size = forceToSquare(size);
	
	switch(taskId) {
		case Evoptool::ALT_BITS: params->task = new AltBits(size); break;	
		case Evoptool::BIN_VAL: params->task = new BinVal(size); break; 
        	case Evoptool::F3D_DECEPTIVE: params->task = new F3Deceptive(size); break;
        	case Evoptool::F3D_DECEPTIVE_BIPOLAR: params->task = new F3DeceptiveBipolar(size); break;
        	case Evoptool::F3D_DECEPTIVE_OVERLAPPING: params->task = new F3DeceptiveOverlapping(size); break;
        	case Evoptool::FOUR_PEAKS: 
        		params->task = new FourPeaks(size, _genSpinButton->get_value_as_int());
        		break; 
        	case Evoptool::LIEPINS_VOSE_FULLY_DECEPTIVE: 
        		params->task = new LiepinsVoseFullyDeceptive(size); 
        		break;
        	case Evoptool::ONE_MAX: params->task = new OneMax(size); break;
        	case Evoptool::ONE_ZERO_MAX: params->task = new OneZeroMax(size); break;
        	case Evoptool::QUADRATIC: params->task = new Quadratic(size); break;
        	case Evoptool::MAX_SAT: params->task = new SatLib((char*) SATLIB_PATH); break;
        	case Evoptool::SIX_PEAKS:
        	        params->task = new SixPeaks(size, _genSpinButton->get_value_as_int());
        		break; 
        	case Evoptool::SUM_VAL: params->task = new SumVal(size); break;
        	case Evoptool::TRAP5: params->task = new Trap5(size); break;
        	case Evoptool::SPIN_GLASS: {
        		char file[100];
        		int s, fn; 
       			
       			genRow = *(_genComboBox->get_active());
			size = genRow[_modelColumns.mcId];
			
       			s = (int) sqrt(size);
       			fn = random() % 10;
       			sprintf(file, "%s%d/%d.dat", SPINGLASS_PATH, s, fn);
       			params->task = new SpinGlass(file, size);
       			break;
       		}
        	case Evoptool::TEXTURE_RESTORATION: {
        		Glib::ustring filename = TEXTURE_RESTORATION_PATH;
        		
        		genRow = *(_genComboBox->get_active());
        		filename.append(genRow[_modelColumns.mcText]);
        		params->task = new TextureRestoration(filename.c_str());
        		params->statisticsOptions |= Evoptool::BEST_AS_IMAGE;
        		break;        	
        	}
        	default: {
        		cerr << "[EvoptoolGUI] Error. Objective function not created. " << endl;
        		break;
        	}
        		
	}
}

/* Initialize resolutors. */
void EvoptoolGUI::initializeResolutors(Evoptool::TestParameters *params) {
	unsigned int i;
	Solver *solver;
	
	for (i = 0; i < _resDecorators.size(); i++) {
		if (_resDecorators[i]->isActive()) {
			solver = _resDecorators[i]->newInstance(params->task, params->averageSamples, params->rng);
			params->solvers.push_back(solver);
		}
	}
}

/* Initialize statistics visualization. */
void EvoptoolGUI::initializeStatisticsVisualization(Evoptool::TestParameters *params) {
	unsigned int i;
	
	/* Erase old tabs except basic graphs. */
	if (_erasableTabs.size() != 0) {
		_statisticsBook->set_current_page(0);
		for (i = 0; i < _erasableTabs.size(); i++)
			delete _erasableTabs[i];
		_erasableTabs.clear();
	}
	
	/* Load best as image tab. */		
	if (params->statisticsOptions & Evoptool::BEST_AS_IMAGE) {
		ImageGridTab *tab;
		int i, elementsNum;
		char **files; 
		char **titles;
		
		elementsNum = params->solvers.size() + 2;
		files = new char *[elementsNum];
		titles = new char *[elementsNum];
		
		files[0] = (char*) TEXTURE_RESTORATION_OPTIMAL_TMP;
		titles[0] = (char*) "Real";
		
		files[1] = (char*) TEXTURE_RESTORATION_OBSERVED_TMP;
		titles[1] = (char*) "Corrupted";
		
		for (i = 2; i < elementsNum; i++) {
			files[i] = new char [100];
			titles[i] = new char [100];
			sprintf(files[i], "%s%d.png", TEXTURE_RESTORATION_TMP, i - 2);
			sprintf(titles[i], "%s", params->solvers[i - 2]->getAlgorithmName().c_str());
		}
		tab = new ImageGridTab(*new Glib::ustring((char*) "BestAsImage"), 
								const_cast<const char**>(files), 
								const_cast<const char**>(titles), elementsNum);		
		_erasableTabs.push_back(tab);	
		_statisticsBook->append_page(*tab, tab->tabName());
		_statisticsBook->show_all_children();
	}
}
	
/* -------------------------------------------- Checker functions ------------------------------------------------ */

/* Checks if the task is correctly defined by the parameters choosed by the user. */
bool EvoptoolGUI::taskValidAssignment() {
	Gtk::TreeModel::iterator taskIt;
	Gtk::TreeModel::Row taskRow;
	int taskId;
	
	if (_taskBox->get_active_row_number() == -1) {
		_statusBar->push("Evoptool: select a task to solve..");
		return false;
	}
	
	taskIt = _taskBox->get_active();
	taskRow = *taskIt;
	
	if (taskIt && taskRow) {
		taskId = taskRow[_modelColumns.mcId];
		switch (taskId) {
			case Evoptool::SPIN_GLASS : 
			case Evoptool::TEXTURE_RESTORATION : 
				if (_genComboBox->get_active_row_number() == -1) {
					_statusBar->push("Evoptool: missing parameters for the task..");
					return false;
				}
				break;
			default: break; 
		}
	} else {
		_statusBar->push("Evoptool:: error, something wrong with pointers..");
		return false;
	}
	
	return true;
}

/* Checks if there is at least one algorithm selected for this test. */
bool EvoptoolGUI::resolutorsValidAssignment() {
	unsigned int i;
	
	for (i = 0; i < _resDecorators.size(); i++) {
		if (_resDecorators[i]->isActive()) return true;
	}
	_statusBar->push("Evoptool: select at least one resolutor..");
	return false;
}

/* Checks if the archive name has a valid assignment. Folder name does not have similar requirement such 
   it is managed by a FileChooser widget. */
bool EvoptoolGUI::exportValidAssignment() {
	if ((_archiveEntry->get_text().size() == 0) && (_saveButton->get_active())) {
		_statusBar->push("Evoptool: enter a valid archive name or shut down save option..");
		return false;
	}
	return true;
}

/* --------------------------------------- Statistics visualization ---------------------------------------------- */

/* This functions updates the statistics visualization of the current page showed in the statistics book. 
 * It works under the assumption that all the files needed to perform the action have just been updated
 * by a previously call to Evoptool low-level function. */
void EvoptoolGUI::updateStatisticsVisualization() {
	unsigned int options, height, width;
	int page;
	
	page = _statisticsBook->get_current_page();
	if (page != -1) {
		switch (page) {
			case 0: options = Evoptool::AVG_GRAPH; break;
			case 1: options = Evoptool::BEST_GRAPH; break;
			case 2: options = Evoptool::WORST_GRAPH; break;
			case 3: options = Evoptool::FCALLS_GRAPH; break;
			case 4: options = Evoptool::BEST_AS_IMAGE; break;
			default: break;
		}
	
		height = _statisticsViewport->get_height() - 50;
		width = _statisticsViewport->get_width() - 55;
			
		_evoptool->signalComputeStatistics().emit(options, height, width);
		if (page != _previousTab) {
			dynamic_cast<StatisticsTab*>(_statisticsBook->get_nth_page(_previousTab))->clearContent();
			_previousTab = page;
		}
	
		dynamic_cast<StatisticsTab*>(_statisticsBook->get_nth_page(page))->refresh();
		_statisticsBook->show_all_children();
	} 
}

/* ----------------------------------- General purpose utility functions ----------------------------------------- */

/* Create a treeModel as specified from the passed parameters. */
void EvoptoolGUI::createTreeModel(BoxEntry *boxEntry, int boxLen, Glib::RefPtr<Gtk::ListStore> &treeModel) {
	int i;
	Gtk::TreeModel::Row row;
		
	treeModel = Gtk::ListStore::create(_modelColumns);
  	
	for (i = 0; i < boxLen; i++) {
		row = *(treeModel->append());
  		row[_modelColumns.mcId] = boxEntry[i].id;
  		row[_modelColumns.mcText] = boxEntry[i].text;
	}
}

/* Returns the greater square number less than value. */
int EvoptoolGUI::forceToSquare(int value) {
	return pow(floor(sqrt(value)), 2);
}
