
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#ifndef __EVOPTOOL_GUI_H_
#define __EVOPTOOL_GUI_H_

#include <gtkmm.h>
#include <libglademm.h>

#include "Evoptool.h"
#include "AlgorithmDecorator.h"
#include "AlgorithmDecorators.h"
#include "StatisticsTab.h"
#include "SimpleGraphTab.h"
#include "ImageGridTab.h"

#define STATISTICS_UPDATE_TIME 400

/* This class defines the Evoptool Graphic User Interface manager. It implements all the operations needed
 * to attach the GUI to the Evoptool engine. */

class EvoptoolGUI : public Gtk::Window {
	/*----------------------------------------------------------------------------------------------------------------*/
	/*						TYPES DEFINITION 						  */
	/*----------------------------------------------------------------------------------------------------------------*/
public: 	
	
	/* This struct defines a container for a combobox entry definition. */
	struct BoxEntry {
		char* text;
		int id;
	};
			
	/* This class defines a model coloums used by combobox widgets. */
	class ModelColumns : public Gtk::TreeModel::ColumnRecord {
        public:
		ModelColumns() { 
			add(mcId); 
                        add(mcText); 
                }

    		Gtk::TreeModelColumn<int> mcId;
    		Gtk::TreeModelColumn<Glib::ustring> mcText;
         };
	
	/*----------------------------------------------------------------------------------------------------------------*/
	/*					   MEMBER VARIABLES DEFINITION	 					  */
	/*----------------------------------------------------------------------------------------------------------------*/
protected: 
	
	/* Command bar references. */
	Gtk::Toolbar *_commandBar;
	Gtk::ToolButton *_runButton;			
	Gtk::ToolButton *_stopButton;			
	Gtk::ToggleToolButton *_saveButton;			
	Gtk::ToggleToolButton *_modeButton;		
	Gtk::FileChooserButton *_folderButton;
	Gtk::Entry *_archiveEntry;		
	Gtk::SpinButton *_averageButton;		
	Gtk::SpinButton *_generationsButton;		
	Gtk::Statusbar *_statusBar;
	
	/* Task bar references. */
        Gtk::Toolbar *_taskBar;
        Gtk::SpinButton *_sizeSpinButton;
        Gtk::SpinButton *_genSpinButton;
        Gtk::ComboBox *_taskBox;
        Gtk::ComboBox *_genComboBox;
        Gtk::Label *_sizeLabel;
        Gtk::Label *_genLabel;
        
        /* Coloum and tree models. */
	ModelColumns _modelColumns;
	Glib::RefPtr<Gtk::ListStore> _taskTreeModel;
	Glib::RefPtr<Gtk::ListStore> _spinGlassTreeModel;
	Glib::RefPtr<Gtk::ListStore> _textureRestorationTreeModel;
	
	/* Resolutors panel references. */
	Gtk::ScrolledWindow *_resolutorsPanel;
	vector<AlgorithmDecorator*> _resDecorators;
	vector<AlgorithmDecorator*> _needSquareSizeResDecorators;	
	
	/* Statistics notebook references and variables. */
	sigc::connection _statisticsRefreshTimer;
	Gtk::Notebook *_statisticsBook;
	Gtk::Viewport *_statisticsViewport;
	Gtk::HPaned *_hpaned;
	vector<StatisticsTab*> _erasableTabs;
	bool _statisticsHandlerEffectiveness;
	int _hpanedPosition;
	int _previousTab; 
	
	/* Logger references. */
	Gtk::TextView *_logView;
	Gtk::TextIter _logIterator;
	Glib::RefPtr<Gtk::TextBuffer> _logBuffer;
	sigc::connection _logHandler;
	
	/* Others references and variables. */
	Evoptool *_evoptool;
	Glib::RefPtr<Gnome::Glade::Xml> _refGlade;
	
	/*----------------------------------------------------------------------------------------------------------------*/
	/*						PUBLIC FUNCTIONS 						  */
	/*----------------------------------------------------------------------------------------------------------------*/
public:
        EvoptoolGUI(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
        virtual ~EvoptoolGUI();
        
        void setEvoptoolReference(Evoptool *tool);
        
	/*----------------------------------------------------------------------------------------------------------------*/
	/* 						PRIVATE FUNCTIONS			       			  */
	/*----------------------------------------------------------------------------------------------------------------*/
private:
	
	/* Initializer functions. */
	void initializeWidgetsAttributes();
	void initializeMainWindow();
	void initializeCommandBar();
	void initializeTaskBar();
	void initializeResolutorsPanel();
	void initializeStatisticsVisualizer();
	void initializeLogVisualizer();
	
	/* Signal handlers functions. */
	void connectSignalHandlers();
	void signalRunTestHandler();
	void signalFinalizeTestHandler();
	void signalTestExecutedHandler();
	void signalNewLogHandler(string message);
	void runButtonHandler();
	void stopButtonHandler();
	void taskBoxHandler();
	bool statisticsRefreshTimer(int timerNumber);
	void statisticsRefreshTimerOnce(int timerNumber);
	void statisticsBookHandler(GtkNotebookPage* page, guint page_num);
	void hpanedHandler(Gdk::Rectangle &allocation);
	
	/* Test initialization functions. */
	void initializerThread();
	void initializeConfiguration(Evoptool::TestParameters *params);
	void initializeTask(Evoptool::TestParameters *params);
	void initializeResolutors(Evoptool::TestParameters *params);
	void initializeStatisticsVisualization(Evoptool::TestParameters *params);
		
	/* Checker functions. */
	bool taskValidAssignment();	
	bool resolutorsValidAssignment();
	bool exportValidAssignment();
	
	/* Statistics visualization. */
	void updateStatisticsVisualization();
	
	/* General purpose utility functions. */
	void createTreeModel(BoxEntry *boxEntry, int boxLen, Glib::RefPtr<Gtk::ListStore> &treeModel);
	int forceToSquare(int value);
};
#endif
