
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "ImageGridTab.h"

/* Class constructor. */
ImageGridTab::ImageGridTab(Glib::ustring &tabName, const char **files, const char **titles, int elementsNumber) :
	StatisticsTab(tabName),
	Gtk::Table(NROW, NCOLUMN, true) {
	initialize(files, titles, elementsNumber);
}

/* Class destructor. */
ImageGridTab::~ImageGridTab() {

}

/* This function refreshes the content of every cell inside the table. */
void ImageGridTab::refresh() {
	unsigned int i;
	
	for (i = 0; i < _cells.size(); i++)
		_cells[i]->refresh();
}

/* This function clears the content of every cell inside the table. */
void ImageGridTab::clearContent() {
	unsigned int i;
	
	for (i = 0; i < _cells.size(); i++)
		_cells[i]->clear();
}

/* This function initializes the table content. */
void ImageGridTab::initialize(const char **files, const char **titles, int elementsNumber) {
	int i, b = 1, t = 0, l = 0, r = 1;
	ImageCell *cell;
	
	set_spacings(2);
	
	//TODO: check elementsNumber if greater than maximum cells number!
	
	for (i = 0; i < elementsNumber; i++) {
		cell = Gtk::manage(new ImageCell(files[i], titles[i]));
		_cells.push_back(cell);
		attach(*cell, l, r, t, b);
		l++;
		r++;
		if (r > NCOLUMN) {
			l = 0;
			r = 1;
			t++;
			b++;
		}
	}
}
