
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#ifndef IMAGEGRIDTAB_H
#define IMAGEGRIDTAB_H

#include "StatisticsTab.h"

#define NROW 3
#define NCOLUMN 4

using namespace std;

/* This class defines the StatisticsTab to visualize the best individuals as images. */

class ImageGridTab : public StatisticsTab, public Gtk::Table {
	
protected: 

	/* This class defines the content of a signle grid cell, which is compose by an Image *
	 * and by a label. */
	class ImageCell : public Gtk::Viewport {
	protected:
		const char* _file;
		Gtk::Image _image;
		Gtk::Label _title;
		Gtk::VBox _box;
	public:
		ImageCell(const char* file, const char* title) :
			Gtk::Viewport(* new Gtk::Adjustment(0.0, 0.0, 100.0, 1.0, 10.0, 10.0), 
				      * new Gtk::Adjustment(0.0, 0.0, 100.0, 1.0, 10.0, 10.0)),
			_file(file),
			_image(file),
			_title(title) {
			_box.pack_start(_image);
			_box.pack_start(_title);	
			add(_box);
		};
		virtual ~ImageCell() { };
		
		/* Refresh the visualized image. */
		void refresh() {
			_image.set(_file);
		};
		
		/* Clear the visualized image. */
		void clear() {
			_image.clear();
		}
	};

	vector<ImageCell*> _cells;
	
public: 
       	ImageGridTab(Glib::ustring &tabName, const char **files, const char **titles, int elementsNumber);
        virtual ~ImageGridTab();

        void refresh();
        void clearContent();
private: 
	void initialize(const char **files, const char **titles, int elementsNumber);
};
#endif
