 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "UnivariateDEUMDecorator.h"
#include "UnivariateDEUM.h"

/* Class constructor. */
UnivariateDEUMDecorator::UnivariateDEUMDecorator(Glib::ustring &title) : AlgorithmDecorator(title) {
	initContent(); 
}

/* Class destructor. */
UnivariateDEUMDecorator::~UnivariateDEUMDecorator() {

}

/* Initializer of the input entries of the decorator. */
void UnivariateDEUMDecorator::initContent() {
	addParameter(POP);
	addParameter(FACTOR);
	addParameter(COOLING_RATE); 
}

/* Returns a Solver object, which is a wrapper around the underlying algorithm. */
Solver* UnivariateDEUMDecorator::newInstance(ObjectiveFunction *task, unsigned int algorithmsNum, gsl_rng *r) {
	Algorithm **algos;
	Solver *solver;
	unsigned int i, pop, it;
	double cool;
	
	pop = _sp_btns[0]->get_value_as_int(); 
	it = _sp_btns[1]->get_value_as_int();
	cool = _sp_btns[2]->get_value();
	
	algos  = new Algorithm*[algorithmsNum];
	for (i = 0; i < algorithmsNum; i++) {
		algos[i] = new UnivariateDEUM(pop, cool, it, r);
		algos[i]->setObjectiveFunction(task);
	}
		
	solver = new Solver(algos, algorithmsNum, UDEUM);
	return solver;
}
