 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "pbilCOMITDecorator.h"
#include "pbilCOMIT.h"

/* Class constructor. */
pbilCOMITDecorator::pbilCOMITDecorator(Glib::ustring &title) : AlgorithmDecorator(title) {
	initContent(); 
}

/* Class destructor. */
pbilCOMITDecorator::~pbilCOMITDecorator() {

}

/* Initializer of the input entries of the decorator. */
void pbilCOMITDecorator::initContent() {
	addParameter(POP);
	addParameter(NUM_SAMPLES);
	addParameter(PERCENTAGE_SELECTION); 
	addParameter(MAX_INFLUENCE);
	addParameter(FAST_SEARCH_ITERATIONS);
	addParameter(MUT_RATE);
	addParameter(GAMMA);
}

/* Returns a Solver object, which is a wrapper around the underlying algorithm. */
Solver* pbilCOMITDecorator::newInstance(ObjectiveFunction *task, unsigned int algorithmsNum, gsl_rng *r) {
	Algorithm **algos;
	Solver *solver;
	unsigned int i, pop, samples, maxInfluence, iterations, percentageSelection;
	double gamma, mutRate;
	
	pop = _sp_btns[0]->get_value_as_int(); 
	samples = _sp_btns[1]->get_value_as_int();
	percentageSelection = _sp_btns[2]->get_value_as_int();
	maxInfluence = _sp_btns[3]->get_value_as_int();
	iterations = _sp_btns[4]->get_value_as_int();
	mutRate = _sp_btns[5]->get_value();
	gamma = _sp_btns[6]->get_value();

	
	algos  = new Algorithm*[algorithmsNum];
	for (i = 0; i < algorithmsNum; i++) {
		algos[i] = new pbilCOMIT(pop, samples, maxInfluence, gamma, mutRate, percentageSelection, iterations, r);
		algos[i]->setObjectiveFunction(task);
	}
		
	solver = new Solver(algos, algorithmsNum, PBILCOMIT);
	return solver;
}
