 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)       * 
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "F3DeceptiveBipolar.h"
#include <stdlib.h>
#include <math.h>

/* Class constructor. */
F3DeceptiveBipolar::F3DeceptiveBipolar(int size) : ObjectiveFunction(size) {

}

/* Class destructor. */
F3DeceptiveBipolar::~F3DeceptiveBipolar() {

}

/* Fitness function. */
double F3DeceptiveBipolar::f(BinaryInstance *bi) {
	int i, len, sum = 0; 
	double normalize; 
	volatile double retValue = 0; 
	
	if (_size % 6 == 0) { normalize = _size / 6; }
	else { normalize = ceil((double) _size / 6); }
	
	if (_size % 6 == 1) { 
		sum += bi->get(_size - 1);
	}
	else if (_size % 6 == 2) { 
		sum += bi->get(_size - 1) + bi->get(_size - 2);	
	}
	else if (_size % 6 == 3) { 
		sum += bi->get(_size - 1) + bi->get(_size - 2) + bi->get(_size - 3);		
	}
	else if (_size % 6 == 4) { 
		sum += bi->get(_size - 1) + bi->get(_size - 2) + bi->get(_size - 3) + bi->get(_size - 4);	
	}
	else if (_size % 6 == 5) { 
		sum += bi->get(_size - 1) + bi->get(_size - 2) + bi->get(_size - 3) + bi->get(_size - 4);
		sum += bi->get(_size -5);	
	}

	if (_size % 6 != 0) {
		if (sum == 0 || sum == 6) retValue +=1;
		else if (sum == 2 || sum == 4) retValue += 0.8;
		else if (sum == 3) retValue += 0.9;	
	}	
		
	len = floor((double) _size / 6) * 6; 
	for (i = 0; i < len; i = i + 6) {
		sum = 0;
		sum += (int)bi->get(i);
		sum += (int)bi->get(i + 1);
		sum += (int)bi->get(i + 2);
		sum += (int)bi->get(i + 3);
		sum += (int)bi->get(i + 4);
		sum += (int)bi->get(i + 5);
		if (sum == 0 || sum == 6) retValue += 1;
		else if (sum == 2 || sum == 4) retValue += 0.8;
		else if (sum == 3) retValue += 0.9;	
	}
	
	retValue = retValue / normalize * 100;
	bi->setFitnessCache(retValue);
	return retValue;
}
