 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/
 
#include <math.h>
#include "FourPeaks.h"

/* Class constructor. */
FourPeaks::FourPeaks(int size, int t) : ObjectiveFunction(size) {
	t_ = t > size ? size - 1 : t;
}

/* Class destructor. */
FourPeaks::~FourPeaks() {

}

/* Fitness function. */
double FourPeaks::f(BinaryInstance *bi) {
	if (bi!=NULL) {
		if(!(bi->validCache())) { 
			/* Fitness cache value not valid */
			int sum = r(bi);
			sum += max(headingOnes(bi),tailingZeros(bi));
			volatile double retValue = (double)sum/((double)(_size + _size - t_)) * 100;
			bi->setFitnessCache(retValue);
			return retValue;
		} else {
			return bi->getFitnessCache();
		}	
	} else {
		cerr << "[FourPeaks::f] binary instance cannot be null" << endl;
		return 0;
	}
}

/* Reward function. */
int FourPeaks::r(BinaryInstance *bi) {
	int rtnValue = 0;
	if (headingOnes(bi) >= t_ && tailingZeros(bi) >= t_) {
		rtnValue = _size;
	}
	return rtnValue;
}

/* Returns heading ones count. */
int FourPeaks::headingOnes(BinaryInstance *bi) {
	int sum = 0;
	int j = 0;
	
	while(j < _size && bi->get(j) == 1) {
		sum++;
		j++;
	}	
	return sum;
}

/* Returns tailing zeros count. */
int FourPeaks::tailingZeros(BinaryInstance *bi) {
	int sum = 0;
	int j = _size - 1;
	
	while( j >= 0 && bi->get(j) == 0) {
		sum++;
		j--;
	}	
	return sum;
}
