
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "SpinGlass.h"
#include <stdlib.h>
#include <fstream>
#include <sstream>

/* Class constructor. */
SpinGlass::SpinGlass(char* filename, int size) : ObjectiveFunction(size) {
	if (!parseFile(filename)) {
		cerr << "[SpinGlass::parseFile] Error during input file parsing!" << endl; 
	}
}

/* Class destructor. */
SpinGlass::~SpinGlass() {
	unsigned int i;
	
	for (i = 0; i < _grid.size(); i++)
		delete _grid[i];
	_grid.clear();
}

/* Fitness function. */
double SpinGlass::f(BinaryInstance *bi) {
	vector<SpinCoupling>::iterator it;
	volatile double fitness = 0;
	
	if (bi->validCache()) return bi->getFitnessCache();
	
	for (unsigned int i=0; i<_grid.size(); i++) {
		fitness += _grid[i]->getCoupling() * bi->getHarmonicEncoding(_grid[i]->getSpin1()) * bi->getHarmonicEncoding(_grid[i]->getSpin2());
	}
	fitness = fitness * 100 / _maxFitness;
	bi->setFitnessCache(fitness); 
	return fitness; 
}

/* Spin Glass instance parser. */
bool SpinGlass::parseFile(char* filename) {
	ifstream file;
	string line; 
	int index, spin1, spin2, coupling; 
	double fitness = 0;
	BinaryInstance *instance; 

	file.open(filename);
	
	do getline(file, line); while (line.find(BEST_BEGIN) == string::npos);
	
	// Read best individual
	instance = new BinaryInstance(_size);
	instance->setAllBits(0);
	while (1) {
		getline(file, line);
		stringstream ss(line);
		if (line.find(BEST_END) != string::npos) break;
		ss >> index; 
		instance->set(index - 1, 1);
	}
	
	do getline(file, line); while (line.find(COUPLING_BEGIN) == string::npos);
	
	// Read coupling constants
	for (int i=0; i<2*_size; i++) {
		getline(file, line);
		stringstream ss(line); 
		ss >> spin1 >> spin2 >> coupling;
		_grid.push_back(new SpinCoupling(spin1 - 1, spin2 - 1, coupling));
	}
	getline(file, line);
	
	// Computing max fitness
	for (unsigned int i=0; i<_grid.size(); i++) {
		fitness += _grid[i]->getCoupling() * instance->getHarmonicEncoding(_grid[i]->getSpin1()) * instance->getHarmonicEncoding(_grid[i]->getSpin2());
	}
	_maxFitness = fitness; 
	delete instance; 
	file.close();
	
	return true; 
}
