
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#ifndef TEXTURERESTORATION_H
#define TEXTURERESTORATION_H

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cblas.h>

#include "ObjectiveFunction.h"

#define CHANNEL_NOISE 100
#define WHITE_NOISE 101
#define PNOISE 0.2
#define BETA 1.0

/* This class defines the Texture Restoration objective function. The constructor of this benchmark 
 * accepts as input a square monochromatic image, given its file name. */

class TextureRestoration : public ObjectiveFunction {
	BinaryInstance* _observed;
	BinaryInstance* _optimal;
	gsl_rng *_r;
	double _optimalValue;
	bool _normalize;
	int **_structure;
	int _structureLength;
	int _noise;
	
public:
        TextureRestoration(const char *filename);
        virtual ~TextureRestoration();

	double f(BinaryInstance* bi);
	void exportOptimalImage(char *file);
	void exportObservedImage(char *file);
	
private: 
	void init(const char *filename);
	void loadStructure();
	BinaryInstance* addNoise(BinaryInstance* original);
	double maxf(BinaryInstance *optimal);
};

#endif
