 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)       * 
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "Trap5.h"
#include <math.h>

/* Class constructor. */
Trap5::Trap5(int size) : ObjectiveFunction(size) {

}

/* Class destructor. */
Trap5::~Trap5() {

}

/* Fitness function. */
double Trap5::f(BinaryInstance *bi) {
	int i, len, sum = 0;
	double normalize;
	volatile double retValue = 0;
	
	if (_size % 5 == 0) {
		normalize = _size;
	} else {
		normalize = floor((double) _size / 5) * 5 + 4;
	}
	
	if (_size % 5 == 1) { 
		sum += bi->get(_size - 1);
		retValue = 4 - sum;
	}
	else if (_size % 5 == 2) { 
		sum += bi->get(_size - 1) + bi->get(_size - 2);	
		retValue = 4 - sum;
	}
	else if (_size % 5 == 3) { 
		sum += bi->get(_size - 1) + bi->get(_size - 2) + bi->get(_size - 3);		
		retValue = 4 - sum;
	}
	else if (_size % 5 == 4) { 
		sum += bi->get(_size - 1) + bi->get(_size - 2) + bi->get(_size - 3) + bi->get(_size - 4);	
		retValue = 4 - sum;
	}
	
	len = floor((double) _size / 5) * 5;
	for (i = 0; i < len; i = i + 5) {
		sum = 0;
		sum += bi->get(i);
		sum += bi->get(i + 1);
		sum += bi->get(i + 2);
		sum += bi->get(i + 3);
		sum += bi->get(i + 4);
		if (sum < 5)
			retValue += 4 - sum;
		else
			retValue += 5;
	}
	
	retValue = retValue / normalize * 100;
	return retValue;
}
