 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "Comparator.h"

/* Class constructor. */
Comparator::Comparator(Algorithm* algo) : _algo(algo) {

}

/* Class destructor. */
Comparator::~Comparator() {

}

/* Compares the fitness value of the arguments exploiting fitness chaching mechanism. */
bool Comparator::operator () (BinaryInstance *a, BinaryInstance *b) {
	volatile double fa, fb;
	
	fa = _algo->fitness(a);
	fb = _algo->fitness(b);
	return (fa>fb);
}

/* Compares a couple of algorithm looking at their average population fitness. */
bool Comparator::operator () (Algorithm *a, Algorithm *b) {
	Statistics *stat;
	
	stat = a->computeStatistics();
	volatile double avga = stat->bestInstance();
	delete stat;
	
	stat = b->computeStatistics();
	volatile double avgb = stat->bestInstance();
	delete stat; 
	
	return (avga>avgb);
}

/* Compares a couple of PermutationInstance. */
bool Comparator::operator () (PermutationInstance *a, PermutationInstance *b) {
	if (!a->validCache() || !b->validCache())
        		cerr << "Comparator::operator() Fitness cache not valid!" << endl;
	volatile double f1 = a->getFitnessCache();
	volatile double f2 = b->getFitnessCache();
	return (f1 > f2);
}
